@@ -23,16 +23,19 @@
 #include <qfile.h>
 #include <qlist.h>
 #include <qpainter.h>
+#include <qprinter.h>
 #include <kaboutdata.h>
 #include <kglobal.h>
 #include <klocale.h>
 #include <kurl.h>
 #include <QBuffer>
 #include <QImageReader>
+#include <QMutex>
 
 #include <okular/core/document.h>
 #include <okular/core/page.h>
 #include <okular/core/area.h>
+#include <okular/core/fileprinter.h>
 
 const int XpsDebug = 4712;
 
@@ -42,19 +45,25 @@ static KAboutData createAboutData()
          "okular_xps",
          "okular_xps",
          ki18n( "XPS Backend" ),
-         "0.2",
+         "0.3",
          ki18n( "An XPS backend" ),
          KAboutData::License_GPL,
          ki18n( "© 2006-2007 Brad Hards\n"
-                "© 2007 Jiri Klement" )
+                "© 2007 Jiri Klement\n"
+                "© 2008 Pino Toscano" )
     );
     aboutData.addAuthor( ki18n( "Brad Hards" ), KLocalizedString(), "bradh@frogmouth.net" );
     aboutData.addAuthor( ki18n( "Jiri Klement" ), KLocalizedString(), "jiri.klement@gmail.com" );
+    aboutData.addAuthor( ki18n( "Pino Toscano" ), KLocalizedString(), "pino@kde.org" );
     return aboutData;
 }
 
 OKULAR_EXPORT_PLUGIN( XpsGenerator, createAboutData() )
 
+Q_DECLARE_METATYPE( QGradient* )
+Q_DECLARE_METATYPE( XpsPathFigure* )
+Q_DECLARE_METATYPE( XpsPathGeometry* )
+
 // From Qt4
 static int hex2int(char hex)
 {
@@ -698,6 +707,24 @@ static void applySpreadStyleToQGradient( const QString &style, QGradient *qgrad
     }
 }
 
+/**
+    Read an UnicodeString
+    \param string the raw value of UnicodeString
+
+    \see XPS specification 5.1.4
+*/
+static QString unicodeString( const QString &raw )
+{
+    QString ret;
+    if ( raw.startsWith( QLatin1String( "{}" ) ) ) {
+        ret = raw.mid( 2 );
+    } else {
+        ret = raw;
+    }
+    return ret;
+}
+
+
 XpsHandler::XpsHandler(XpsPage *page): m_page(page)
 {
     m_painter = NULL;
@@ -705,13 +732,11 @@ XpsHandler::XpsHandler(XpsPage *page): m_page(page)
 
 XpsHandler::~XpsHandler()
 {
-    delete m_painter;
 }
 
 bool XpsHandler::startDocument()
 {
     kDebug(XpsDebug) << "start document" << m_page->m_fileName ;
-    m_page->m_pageImage->fill( qRgba( 255, 255, 255, 255 ) );
 
     XpsRenderNode node;
     node.name = "document";
@@ -731,7 +756,6 @@ bool XpsHandler::startElement( const QString &nameSpace,
     XpsRenderNode node;
     node.name = localName;
     node.attributes = atts;
-    node.data = NULL;
     processStartElement( node );
     m_nodes.push(node);
 
@@ -798,10 +822,9 @@ void XpsHandler::processGlyph( XpsRenderNode &node )
     QBrush brush;
     att = node.attributes.value("Fill");
     if (att.isEmpty()) {
-        XpsFill * data = (XpsFill *)node.getChildData( "Glyphs.Fill" );
-        if (data != NULL) {
-            brush = * data;
-        delete data;
+        QVariant data = node.getChildData( "Glyphs.Fill" );
+        if (data.canConvert<QBrush>()) {
+            brush = data.value<QBrush>();
         } else {
             // no "Fill" attribute and no "Glyphs.Fill" child, so show nothing
             // (see XPS specs, 5.10)
@@ -847,7 +870,7 @@ void XpsHandler::processGlyph( XpsRenderNode &node )
         }
     }
 
-    m_painter->drawText( origin, node.attributes.value("UnicodeString") );
+    m_painter->drawText( origin, unicodeString( node.attributes.value( "UnicodeString" ) ) );
     // kDebug(XpsDebug) << "Glyphs: " << atts.value("Fill") << ", " << atts.value("FontUri");
     // kDebug(XpsDebug) << "    Origin: " << atts.value("OriginX") << "," << atts.value("OriginY");
     // kDebug(XpsDebug) << "    Unicode: " << atts.value("UnicodeString");
@@ -897,10 +920,9 @@ void XpsHandler::processImageBrush( XpsRenderNode &node )
     QMatrix viewportMatrix;
     att = node.attributes.value( "Transform" );
     if ( att.isEmpty() ) {
-        XpsMatrixTransform * data = (XpsMatrixTransform *)node.getChildData( "ImageBrush.Transform" );
-        if (data != NULL) {
-            viewportMatrix = *data;
-            delete data;
+        QVariant data = node.getChildData( "ImageBrush.Transform" );
+        if (data.canConvert<QMatrix>()) {
+            viewportMatrix = data.value<QMatrix>();
         } else {
             viewportMatrix = QMatrix();
         }
@@ -913,7 +935,7 @@ void XpsHandler::processImageBrush( XpsRenderNode &node )
     brush = QBrush( image );
     brush.setMatrix( viewboxMatrix.inverted() * viewportMatrix );
 
-    node.data = new QBrush( brush );
+    node.data = qVariantFromValue( brush );
 }
 
 void XpsHandler::processPath( XpsRenderNode &node )
@@ -924,9 +946,10 @@ void XpsHandler::processPath( XpsRenderNode &node )
     m_painter->save();
 
     QString att;
+    QVariant data;
 
     // Get path
-    XpsPathGeometry * pathdata = (XpsPathGeometry *)node.getChildData( "Path.Data" );
+    XpsPathGeometry * pathdata = node.getChildData( "Path.Data" ).value< XpsPathGeometry * >();
     att = node.attributes.value( "Data" );
     if (! att.isEmpty() ) {
         QPainterPath path = parseAbbreviatedPathData( att );
@@ -946,12 +969,9 @@ void XpsHandler::processPath( XpsRenderNode &node )
     if (! att.isEmpty() ) {
         brush = parseRscRefColorForBrush( att );
     } else {
-        XpsFill * data = (XpsFill *)node.getChildData( "Path.Fill" );
-        if (data != NULL) {
-            brush = *data;
-        delete data;
-        } else {
-            brush = QBrush();
+        data = node.getChildData( "Path.Fill" );
+        if (data.canConvert<QBrush>()) {
+            brush = data.value<QBrush>();
         }
     }
     m_painter->setBrush( brush );
@@ -962,10 +982,9 @@ void XpsHandler::processPath( XpsRenderNode &node )
     if  (! att.isEmpty() ) {
         pen = parseRscRefColorForPen( att );
     } else {
-        XpsFill * data = (XpsFill *)node.getChildData( "Path.Stroke" );
-        if (data != NULL) {
-            pen.setBrush( *data );
-            delete data;
+        data = node.getChildData( "Path.Stroke" );
+        if (data.canConvert<QBrush>()) {
+            pen.setBrush( data.value<QBrush>() );
         }
     }
     att = node.attributes.value( "StrokeThickness" );
@@ -1074,11 +1093,9 @@ void XpsHandler::processPathGeometry( XpsRenderNode &node )
 {
     XpsPathGeometry * geom = new XpsPathGeometry();
 
-    node.data = 0;
-
     Q_FOREACH ( const XpsRenderNode &child, node.children ) {
-        if ( child.data ) {
-            XpsPathFigure *figure = (XpsPathFigure *)child.data;
+        if ( child.data.canConvert<XpsPathFigure *>() ) {
+            XpsPathFigure *figure = child.data.value<XpsPathFigure *>();
             geom->paths.append( figure );
         }
     }
@@ -1105,7 +1122,7 @@ void XpsHandler::processPathGeometry( XpsRenderNode &node )
     }
 
     if ( !geom->paths.isEmpty() ) {
-        node.data = geom;
+        node.data = qVariantFromValue( geom );
     } else {
         delete geom;
     }
@@ -1118,8 +1135,6 @@ void XpsHandler::processPathFigure( XpsRenderNode &node )
     QString att;
     QPainterPath path;
 
-    node.data = 0;
-
     att = node.attributes.value( "StartPoint" );
     if ( !att.isEmpty() ) {
         QPointF point = getPointFromString( att );
@@ -1196,7 +1211,7 @@ void XpsHandler::processPathFigure( XpsRenderNode &node )
     }
 
     if ( !path.isEmpty() ) {
-        node.data = new XpsPathFigure( path, isFilled );
+        node.data = qVariantFromValue( new XpsPathFigure( path, isFilled ) );
     }
 }
 
@@ -1230,12 +1245,11 @@ void XpsHandler::processEndElement( XpsRenderNode &node )
         processPath( node );
     } else if (node.name == "MatrixTransform") {
         //TODO Ignoring x:key
-        node.data = new QMatrix ( attsToMatrix( node.attributes.value( "Matrix" ) ) );
+        node.data = qVariantFromValue( QMatrix( attsToMatrix( node.attributes.value( "Matrix" ) ) ) );
     } else if ((node.name == "Canvas.RenderTransform") || (node.name == "Glyphs.RenderTransform") || (node.name == "Path.RenderTransform"))  {
-        XpsMatrixTransform * data = (XpsMatrixTransform *)node.getRequiredChildData( "MatrixTransform" );
-        if (data != NULL) {
-            m_painter->setWorldMatrix( *data, true );
-        delete data;
+        QVariant data = node.getRequiredChildData( "MatrixTransform" );
+        if (data.canConvert<QMatrix>()) {
+            m_painter->setWorldMatrix( data.value<QMatrix>(), true );
         }
     } else if (node.name == "Canvas") {
         m_painter->restore();
@@ -1245,37 +1259,37 @@ void XpsHandler::processEndElement( XpsRenderNode &node )
         processStroke( node );
     } else if (node.name == "SolidColorBrush") {
         //TODO Ignoring opacity, x:key
-        node.data = new QBrush( QColor (hexToRgba( node.attributes.value( "Color" ).toLatin1() ) ) );
+        node.data = qVariantFromValue( QBrush( QColor( hexToRgba( node.attributes.value( "Color" ).toLatin1() ) ) ) );
     } else if (node.name == "ImageBrush") {
         processImageBrush( node );
     } else if (node.name == "ImageBrush.Transform") {
         node.data = node.getRequiredChildData( "MatrixTransform" );
     } else if (node.name == "LinearGradientBrush") {
         XpsRenderNode * gradients = node.findChild( "LinearGradientBrush.GradientStops" );
-        if ( gradients && gradients->data ) {
+        if ( gradients && gradients->data.canConvert< QGradient * >() ) {
             QPointF start = getPointFromString( node.attributes.value( "StartPoint" ) );
             QPointF end = getPointFromString( node.attributes.value( "EndPoint" ) );
-            QLinearGradient * qgrad = static_cast< QLinearGradient * >( gradients->data );
+            QLinearGradient * qgrad = static_cast< QLinearGradient * >( gradients->data.value< QGradient * >() );
             qgrad->setStart( start );
             qgrad->setFinalStop( end );
             applySpreadStyleToQGradient( node.attributes.value( "SpreadMethod" ), qgrad );
-            node.data = new QBrush( *qgrad );
+            node.data = qVariantFromValue( QBrush( *qgrad ) );
             delete qgrad;
         }
     } else if (node.name == "RadialGradientBrush") {
         XpsRenderNode * gradients = node.findChild( "RadialGradientBrush.GradientStops" );
-        if ( gradients && gradients->data ) {
+        if ( gradients && gradients->data.canConvert< QGradient * >() ) {
             QPointF center = getPointFromString( node.attributes.value( "Center" ) );
             QPointF origin = getPointFromString( node.attributes.value( "GradientOrigin" ) );
             double radiusX = node.attributes.value( "RadiusX" ).toDouble();
             double radiusY = node.attributes.value( "RadiusY" ).toDouble();
-            QRadialGradient * qgrad = static_cast< QRadialGradient * >( gradients->data );
+            QRadialGradient * qgrad = static_cast< QRadialGradient * >( gradients->data.value< QGradient * >() );
             qgrad->setCenter( center );
             qgrad->setFocalPoint( origin );
             // TODO what in case of different radii?
             qgrad->setRadius( qMin( radiusX, radiusY ) );
             applySpreadStyleToQGradient( node.attributes.value( "SpreadMethod" ), qgrad );
-            node.data = new QBrush( *qgrad );
+            node.data = qVariantFromValue( QBrush( *qgrad ) );
             delete qgrad;
         }
     } else if (node.name == "LinearGradientBrush.GradientStops") {
@@ -1289,7 +1303,7 @@ void XpsHandler::processEndElement( XpsRenderNode &node )
         if ( !gradients.isEmpty() ) {
             QLinearGradient * qgrad = new QLinearGradient();
             addXpsGradientsToQGradient( gradients, qgrad );
-            node.data = qgrad;
+            node.data = qVariantFromValue< QGradient * >( qgrad );
         }
     } else if (node.name == "RadialGradientBrush.GradientStops") {
         QList<XpsGradient> gradients;
@@ -1302,7 +1316,7 @@ void XpsHandler::processEndElement( XpsRenderNode &node )
         if ( !gradients.isEmpty() ) {
             QRadialGradient * qgrad = new QRadialGradient();
             addXpsGradientsToQGradient( gradients, qgrad );
-            node.data = qgrad;
+            node.data = qVariantFromValue< QGradient * >( qgrad );
         }
     } else if (node.name == "PathFigure") {
         processPathFigure( node );
@@ -1361,22 +1375,9 @@ bool XpsPage::renderToImage( QImage *p )
         m_pageIsRendered = false;
     }
     if (! m_pageIsRendered) {
-        XpsHandler *handler = new XpsHandler( this );
-        handler->m_painter = new QPainter( m_pageImage );
-        handler->m_painter->setWorldMatrix(QMatrix().scale((qreal)p->size().width() / size().width(), (qreal)p->size().height() / size().height()));
-        QXmlSimpleReader *parser = new QXmlSimpleReader();
-        parser->setContentHandler( handler );
-        parser->setErrorHandler( handler );
-        const KZipFileEntry* pageFile = static_cast<const KZipFileEntry *>(m_file->xpsArchive()->directory()->entry( m_fileName ));
-        QByteArray data = readFileOrDirectoryParts( pageFile );
-        QBuffer * buffer = new QBuffer(&data);
-        QXmlInputSource *source = new QXmlInputSource(buffer);
-        bool ok = parser->parse( source );
-        kDebug(XpsDebug) << "Parse result: " << ok;
-        delete source;
-        delete parser;
-        delete handler;
-        delete buffer;
+        m_pageImage->fill( qRgba( 255, 255, 255, 255 ) );
+        QPainter painter( m_pageImage );
+        renderToPainter( &painter );
         m_pageIsRendered = true;
     }
 
@@ -1385,6 +1386,24 @@ bool XpsPage::renderToImage( QImage *p )
     return true;
 }
 
+bool XpsPage::renderToPainter( QPainter *painter )
+{
+    XpsHandler handler( this );
+    handler.m_painter = painter;
+    handler.m_painter->setWorldMatrix(QMatrix().scale((qreal)painter->device()->width() / size().width(), (qreal)painter->device()->height() / size().height()));
+    QXmlSimpleReader parser;
+    parser.setContentHandler( &handler );
+    parser.setErrorHandler( &handler );
+    const KZipFileEntry* pageFile = static_cast<const KZipFileEntry *>(m_file->xpsArchive()->directory()->entry( m_fileName ));
+    QByteArray data = readFileOrDirectoryParts( pageFile );
+    QBuffer buffer( &data );
+    QXmlInputSource source( &buffer );
+    bool ok = parser.parse( source );
+    kDebug(XpsDebug) << "Parse result: " << ok;
+
+    return true;
+}
+
 QSizeF XpsPage::size() const
 {
     return m_pageSize;
@@ -1565,7 +1584,7 @@ Okular::TextPage* XpsPage::textPage()
                 if (!att.isEmpty()) {
                     matrix = parseRscRefMatrix( att ) * matrix;
                 }
-                QString text =  glyphsAtts.value( "UnicodeString" ).toString();
+                QString text = unicodeString( glyphsAtts.value( "UnicodeString" ).toString() );
 
                 // Get font (doesn't work well because qt doesn't allow to load font from file)
                 QFont font = m_file->getFontByName( glyphsAtts.value( "FontUri" ).toString(),
@@ -1991,6 +2010,8 @@ XpsGenerator::XpsGenerator( QObject *parent, const QVariantList &args )
   : Okular::Generator( parent, args ), m_xpsFile( 0 )
 {
     setFeature( TextExtraction );
+    setFeature( PrintNative );
+    setFeature( PrintToFile );
     // activate the threaded rendering iif:
     // 1) QFontDatabase says so
     // 2) Qt >= 4.4.0 (see Trolltech task ID: 169502)
@@ -1999,6 +2020,7 @@ XpsGenerator::XpsGenerator( QObject *parent, const QVariantList &args )
     if ( QFontDatabase::supportsThreadedFontRendering() )
         setFeature( Threaded );
 #endif
+    userMutex();
 }
 
 XpsGenerator::~XpsGenerator()
@@ -2039,6 +2061,7 @@ bool XpsGenerator::doCloseDocument()
 
 QImage XpsGenerator::image( Okular::PixmapRequest * request )
 {
+    QMutexLocker lock( userMutex() );
     QSize size( (int)request->width(), (int)request->height() );
     QImage image( size, QImage::Format_RGB32 );
     XpsPage *pageToRender = m_xpsFile->page( request->page()->number() );
@@ -2048,6 +2071,7 @@ QImage XpsGenerator::image( Okular::PixmapRequest * request )
 
 Okular::TextPage* XpsGenerator::textPage( Okular::Page * page )
 {
+    QMutexLocker lock( userMutex() );
     XpsPage * xpsPage = m_xpsFile->page( page->number() );
     return xpsPage->textPage();
 }
@@ -2107,6 +2131,26 @@ bool XpsGenerator::exportTo( const QString &fileName, const Okular::ExportFormat
     return false;
 }
 
+bool XpsGenerator::print( QPrinter &printer )
+{
+    QList<int> pageList = Okular::FilePrinter::pageList( printer, document()->pages(),
+                                                         document()->bookmarkedPageList() );
+
+    QPainter painter( &printer );
+
+    for ( int i = 0; i < pageList.count(); ++i )
+    {
+        if ( i != 0 )
+            printer.newPage();
+
+        const int page = pageList.at( i ) - 1;
+        XpsPage *pageToRender = m_xpsFile->page( page );
+        pageToRender->renderToPainter( &painter );
+    }
+
+    return true;
+}
+
 XpsRenderNode * XpsRenderNode::findChild( const QString &name )
 {
     for (int i = 0; i < children.size(); i++) {
@@ -2118,22 +2162,22 @@ XpsRenderNode * XpsRenderNode::findChild( const QString &name )
     return NULL;
 }
 
-void * XpsRenderNode::getRequiredChildData( const QString &name )
+QVariant XpsRenderNode::getRequiredChildData( const QString &name )
 {
     XpsRenderNode * child = findChild( name );
     if (child == NULL) {
         kDebug(XpsDebug) << "Required element " << name << " is missing in " << this->name;
-        return NULL;
+        return QVariant();
     }
 
     return child->data;
 }
 
-void * XpsRenderNode::getChildData( const QString &name )
+QVariant XpsRenderNode::getChildData( const QString &name )
 {
     XpsRenderNode * child = findChild( name );
     if (child == NULL) {
-        return NULL;
+        return QVariant();
     } else {
         return child->data;
     }
